clear; set(0,'defaultaxesfontsize',20); format long
%%% p2.m smoothing problem for the deterministic logistic map (Ex. 1.4)
%% setup

J=1000;% number of steps
r=2;% dynamics determined by r
gamma=0.1;% observational noise variance is gamma^2
C0=0.01;% prior initial condition variance
m0=0.7;% prior initial condition mean
sd=1;rng(sd);% choose random number seed

%% truth

vt=zeros(J+1,1); y=zeros(J,1);% preallocate space to save time
vt(1)=0.1;% truth initial condition
for j=1:J
    % can be replaced by Psi for each problem
    vt(j+1)=r*vt(j)*(1-vt(j));% create truth
    y(j)=vt(j+1)+gamma*randn;% create data
end

%% solution

v0=[0.01:0.0005:0.99];% construct vector of different initial data
Phidet=zeros(length(v0),1);Idet=Phidet;Jdet=Phidet;% preallocate space
vv=zeros(J,1);% preallocate space 
% loop through initial conditions vv0, and compute log posterior I0(vv0) 
for j=1:length(v0) 
   vv(1)=v0(j); Jdet(j)=1/2/C0*(v0(j)-m0)^2;% background penalization 
     for i=1:J
        vv(i+1)=r*vv(i)*(1-vv(i)); 
        Phidet(j)=Phidet(j)+1/2/gamma^2*(y(i)-vv(i+1))^2;% misfit functional
     end 
    Idet(j)=Phidet(j)+Jdet(j);
end

constant=trapz(v0,exp(-Idet));% approximate normalizing constant
P=exp(-Idet)/constant;% normalize posterior distribution
prior=normpdf(v0,m0,sqrt(C0));% calculate prior distribution

figure(1),plot(v0,prior,'k','LineWidth',2)
hold on, plot(v0,P,'r--','LineWidth',2), xlabel 'v_0',
legend 'prior' J=10^3  
